# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class V1VisionStepResultsItemExpirationDate(UniversalBaseModel):
    date_offset: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="dateOffset"), pydantic.Field(alias="dateOffset")
    ] = None
    found_date: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="foundDate"), pydantic.Field(alias="foundDate")
    ] = None
    match_date: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="matchDate"), pydantic.Field(alias="matchDate")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .in_app_notification_options_object_response_body import InAppNotificationOptionsObjectResponseBody
from .push_notification_options_object_response_body import PushNotificationOptionsObjectResponseBody


class DriverAppNotificationObjectResponseBody(UniversalBaseModel):
    """
    Driver app notification settings
    """

    in_app_notification_options: typing_extensions.Annotated[
        typing.Optional[InAppNotificationOptionsObjectResponseBody],
        FieldMetadata(alias="inAppNotificationOptions"),
        pydantic.Field(alias="inAppNotificationOptions"),
    ] = None
    push_notification_options: typing_extensions.Annotated[
        typing.Optional[PushNotificationOptionsObjectResponseBody],
        FieldMetadata(alias="pushNotificationOptions"),
        pydantic.Field(alias="pushNotificationOptions"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .form_submission_request_table_row_object_request_body import FormSubmissionRequestTableRowObjectRequestBody


class FormSubmissionRequestTableValueObjectRequestBody(UniversalBaseModel):
    """
    The value of a table form input field. Only valid for table form input fields.
    """

    rows: typing.List[FormSubmissionRequestTableRowObjectRequestBody] = pydantic.Field()
    """
    List of rows in the table.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class IftaDetailJobArgsResponseBody(UniversalBaseModel):
    """
    The arguments used to create this job.
    """

    end_hour: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="endHour"),
        pydantic.Field(alias="endHour", description="The `endHour` used to create this job."),
    ]
    start_hour: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="startHour"),
        pydantic.Field(alias="startHour", description="The `startHour` used to create this job."),
    ]
    vehicle_ids: typing_extensions.Annotated[
        typing.List[int],
        FieldMetadata(alias="vehicleIds"),
        pydantic.Field(
            alias="vehicleIds",
            description="The vehicleIds used to create this job. Determined by the filter list of `vehicleIds`, `vehicleTagIds`, and `vehicleParentTagIds` parameters (if provided).",
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .ambient_temperature_response_body import AmbientTemperatureResponseBody
from .camera_connector_disconected_response_body import CameraConnectorDisconectedResponseBody
from .camera_stream_issue_response_body import CameraStreamIssueResponseBody
from .cell_signal_loss_response_body import CellSignalLossResponseBody
from .cloud_backup_upload_issue_response_body import CloudBackupUploadIssueResponseBody
from .dashcam_disconnected_response_body import DashcamDisconnectedResponseBody
from .data_input_value_response_body import DataInputValueResponseBody
from .device_movement_data_response_body import DeviceMovementDataResponseBody
from .device_movement_stopped_data_response_body import DeviceMovementStoppedDataResponseBody
from .door_open_response_body import DoorOpenResponseBody
from .driver_app_sign_in_response_body import DriverAppSignInResponseBody
from .driver_app_sign_out_response_body import DriverAppSignOutResponseBody
from .driver_document_submitted_response_body import DriverDocumentSubmittedResponseBody
from .driver_message_received_response_body import DriverMessageReceivedResponseBody
from .driver_message_sent_response_body import DriverMessageSentResponseBody
from .driver_recorded_response_body import DriverRecordedResponseBody
from .engine_idle_data_response_body import EngineIdleDataResponseBody
from .engine_off_response_body import EngineOffResponseBody
from .engine_on_response_body import EngineOnResponseBody
from .form_submitted_response_body import FormSubmittedResponseBody
from .form_updated_response_body import FormUpdatedResponseBody
from .fuel_level_percentage_response_body import FuelLevelPercentageResponseBody
from .gateway_disconnected_response_body import GatewayDisconnectedResponseBody
from .gateway_with_vehicle_tiny_response_response_body import GatewayWithVehicleTinyResponseResponseBody
from .gps_signal_loss_response_body import GpsSignalLossResponseBody
from .harsh_event_data_response_body import HarshEventDataResponseBody
from .hos_violation_data_response_body import HosViolationDataResponseBody
from .inactivity_response_body import InactivityResponseBody
from .inside_geofence_data_response_body import InsideGeofenceDataResponseBody
from .issue_created_response_body import IssueCreatedResponseBody
from .jamming_detected_response_body import JammingDetectedResponseBody
from .missing_dvir_past_due_response_body import MissingDvirPastDueResponseBody
from .motion_detected_response_body import MotionDetectedResponseBody
from .out_of_route_response_body import OutOfRouteResponseBody
from .outside_geofence_data_response_body import OutsideGeofenceDataResponseBody
from .panic_button_response_body import PanicButtonResponseBody
from .person_detected_response_body import PersonDetectedResponseBody
from .reading_trigger_details_response_body import ReadingTriggerDetailsResponseBody
from .reefer_temperature_response_body import ReeferTemperatureResponseBody
from .route_stop_details_object_response_body import RouteStopDetailsObjectResponseBody
from .route_stop_eta_response_body import RouteStopEtaResponseBody
from .scheduled_maintenance_by_engine_hours_response_body import ScheduledMaintenanceByEngineHoursResponseBody
from .scheduled_maintenance_data_response_body import ScheduledMaintenanceDataResponseBody
from .scheduled_maintenance_odometer_data_response_body import ScheduledMaintenanceOdometerDataResponseBody
from .severe_speeding_started_response_object_response_body import SevereSpeedingStartedResponseObjectResponseBody
from .site_gateway_disconnected_response_body import SiteGatewayDisconnectedResponseBody
from .speed_data_response_body import SpeedDataResponseBody
from .sudden_fuel_level_drop_response_body import SuddenFuelLevelDropResponseBody
from .sudden_fuel_level_rise_response_body import SuddenFuelLevelRiseResponseBody
from .tampering_detected_response_body import TamperingDetectedResponseBody
from .tire_faults_response_body import TireFaultsResponseBody
from .unassigned_driving_data_response_body import UnassignedDrivingDataResponseBody
from .vehicle_battery_voltage_response_body import VehicleBatteryVoltageResponseBody
from .vehicle_def_level_percentage_response_body import VehicleDefLevelPercentageResponseBody
from .vehicle_detected_response_body import VehicleDetectedResponseBody
from .vehicle_faults_response_body import VehicleFaultsResponseBody
from .worker_safety_sos_data_response_body import WorkerSafetySosDataResponseBody
from .workflow_dvir_submitted_response_object_response_body import WorkflowDvirSubmittedResponseObjectResponseBody
from .workflow_geofence_event_response_object_response_body import WorkflowGeofenceEventResponseObjectResponseBody


class WorkflowIncidentDetailsObjectResponseBody(UniversalBaseModel):
    """
    Object representing the granular details of the condition. These details will vary depending on the condition.
    """

    ambient_temperature: typing_extensions.Annotated[
        typing.Optional[AmbientTemperatureResponseBody],
        FieldMetadata(alias="ambientTemperature"),
        pydantic.Field(alias="ambientTemperature"),
    ] = None
    camera_connector_disconected: typing_extensions.Annotated[
        typing.Optional[CameraConnectorDisconectedResponseBody],
        FieldMetadata(alias="cameraConnectorDisconected"),
        pydantic.Field(alias="cameraConnectorDisconected"),
    ] = None
    camera_stream_issue: typing_extensions.Annotated[
        typing.Optional[CameraStreamIssueResponseBody],
        FieldMetadata(alias="cameraStreamIssue"),
        pydantic.Field(alias="cameraStreamIssue"),
    ] = None
    cell_signal_loss: typing_extensions.Annotated[
        typing.Optional[CellSignalLossResponseBody],
        FieldMetadata(alias="cellSignalLoss"),
        pydantic.Field(alias="cellSignalLoss"),
    ] = None
    cloud_backup_upload_issue: typing_extensions.Annotated[
        typing.Optional[CloudBackupUploadIssueResponseBody],
        FieldMetadata(alias="cloudBackupUploadIssue"),
        pydantic.Field(alias="cloudBackupUploadIssue"),
    ] = None
    dashcam_disconnected: typing_extensions.Annotated[
        typing.Optional[DashcamDisconnectedResponseBody],
        FieldMetadata(alias="dashcamDisconnected"),
        pydantic.Field(alias="dashcamDisconnected"),
    ] = None
    data_input_value: typing_extensions.Annotated[
        typing.Optional[DataInputValueResponseBody],
        FieldMetadata(alias="dataInputValue"),
        pydantic.Field(alias="dataInputValue"),
    ] = None
    device_movement: typing_extensions.Annotated[
        typing.Optional[DeviceMovementDataResponseBody],
        FieldMetadata(alias="deviceMovement"),
        pydantic.Field(alias="deviceMovement"),
    ] = None
    device_movement_stopped: typing_extensions.Annotated[
        typing.Optional[DeviceMovementStoppedDataResponseBody],
        FieldMetadata(alias="deviceMovementStopped"),
        pydantic.Field(alias="deviceMovementStopped"),
    ] = None
    door_open: typing_extensions.Annotated[
        typing.Optional[DoorOpenResponseBody], FieldMetadata(alias="doorOpen"), pydantic.Field(alias="doorOpen")
    ] = None
    driver_app_sign_in: typing_extensions.Annotated[
        typing.Optional[DriverAppSignInResponseBody],
        FieldMetadata(alias="driverAppSignIn"),
        pydantic.Field(alias="driverAppSignIn"),
    ] = None
    driver_app_sign_out: typing_extensions.Annotated[
        typing.Optional[DriverAppSignOutResponseBody],
        FieldMetadata(alias="driverAppSignOut"),
        pydantic.Field(alias="driverAppSignOut"),
    ] = None
    driver_document_submitted: typing_extensions.Annotated[
        typing.Optional[DriverDocumentSubmittedResponseBody],
        FieldMetadata(alias="driverDocumentSubmitted"),
        pydantic.Field(alias="driverDocumentSubmitted"),
    ] = None
    driver_message_received: typing_extensions.Annotated[
        typing.Optional[DriverMessageReceivedResponseBody],
        FieldMetadata(alias="driverMessageReceived"),
        pydantic.Field(alias="driverMessageReceived"),
    ] = None
    driver_message_sent: typing_extensions.Annotated[
        typing.Optional[DriverMessageSentResponseBody],
        FieldMetadata(alias="driverMessageSent"),
        pydantic.Field(alias="driverMessageSent"),
    ] = None
    driver_recorded: typing_extensions.Annotated[
        typing.Optional[DriverRecordedResponseBody],
        FieldMetadata(alias="driverRecorded"),
        pydantic.Field(alias="driverRecorded"),
    ] = None
    dvir_submitted_device: typing_extensions.Annotated[
        typing.Optional[WorkflowDvirSubmittedResponseObjectResponseBody],
        FieldMetadata(alias="dvirSubmittedDevice"),
        pydantic.Field(alias="dvirSubmittedDevice"),
    ] = None
    engine_idle: typing_extensions.Annotated[
        typing.Optional[EngineIdleDataResponseBody],
        FieldMetadata(alias="engineIdle"),
        pydantic.Field(alias="engineIdle"),
    ] = None
    engine_off: typing_extensions.Annotated[
        typing.Optional[EngineOffResponseBody], FieldMetadata(alias="engineOff"), pydantic.Field(alias="engineOff")
    ] = None
    engine_on: typing_extensions.Annotated[
        typing.Optional[EngineOnResponseBody], FieldMetadata(alias="engineOn"), pydantic.Field(alias="engineOn")
    ] = None
    form_submitted: typing_extensions.Annotated[
        typing.Optional[FormSubmittedResponseBody],
        FieldMetadata(alias="formSubmitted"),
        pydantic.Field(alias="formSubmitted"),
    ] = None
    form_updated: typing_extensions.Annotated[
        typing.Optional[FormUpdatedResponseBody],
        FieldMetadata(alias="formUpdated"),
        pydantic.Field(alias="formUpdated"),
    ] = None
    fuel_level_percentage: typing_extensions.Annotated[
        typing.Optional[FuelLevelPercentageResponseBody],
        FieldMetadata(alias="fuelLevelPercentage"),
        pydantic.Field(alias="fuelLevelPercentage"),
    ] = None
    gateway_disconnected: typing_extensions.Annotated[
        typing.Optional[GatewayDisconnectedResponseBody],
        FieldMetadata(alias="gatewayDisconnected"),
        pydantic.Field(alias="gatewayDisconnected"),
    ] = None
    gateway_unplugged: typing_extensions.Annotated[
        typing.Optional[GatewayWithVehicleTinyResponseResponseBody],
        FieldMetadata(alias="gatewayUnplugged"),
        pydantic.Field(alias="gatewayUnplugged"),
    ] = None
    geofence_entry: typing_extensions.Annotated[
        typing.Optional[WorkflowGeofenceEventResponseObjectResponseBody],
        FieldMetadata(alias="geofenceEntry"),
        pydantic.Field(alias="geofenceEntry"),
    ] = None
    geofence_exit: typing_extensions.Annotated[
        typing.Optional[WorkflowGeofenceEventResponseObjectResponseBody],
        FieldMetadata(alias="geofenceExit"),
        pydantic.Field(alias="geofenceExit"),
    ] = None
    gps_signal_loss: typing_extensions.Annotated[
        typing.Optional[GpsSignalLossResponseBody],
        FieldMetadata(alias="gpsSignalLoss"),
        pydantic.Field(alias="gpsSignalLoss"),
    ] = None
    harsh_event: typing_extensions.Annotated[
        typing.Optional[HarshEventDataResponseBody],
        FieldMetadata(alias="harshEvent"),
        pydantic.Field(alias="harshEvent"),
    ] = None
    hos_violation: typing_extensions.Annotated[
        typing.Optional[HosViolationDataResponseBody],
        FieldMetadata(alias="hosViolation"),
        pydantic.Field(alias="hosViolation"),
    ] = None
    inactivity: typing.Optional[InactivityResponseBody] = None
    inside_geofence: typing_extensions.Annotated[
        typing.Optional[InsideGeofenceDataResponseBody],
        FieldMetadata(alias="insideGeofence"),
        pydantic.Field(alias="insideGeofence"),
    ] = None
    issue_created: typing_extensions.Annotated[
        typing.Optional[IssueCreatedResponseBody],
        FieldMetadata(alias="issueCreated"),
        pydantic.Field(alias="issueCreated"),
    ] = None
    jamming_detected: typing_extensions.Annotated[
        typing.Optional[JammingDetectedResponseBody],
        FieldMetadata(alias="jammingDetected"),
        pydantic.Field(alias="jammingDetected"),
    ] = None
    missing_dvir_past_due: typing_extensions.Annotated[
        typing.Optional[MissingDvirPastDueResponseBody],
        FieldMetadata(alias="missingDvirPastDue"),
        pydantic.Field(alias="missingDvirPastDue"),
    ] = None
    motion_detected: typing_extensions.Annotated[
        typing.Optional[MotionDetectedResponseBody],
        FieldMetadata(alias="motionDetected"),
        pydantic.Field(alias="motionDetected"),
    ] = None
    out_of_route: typing_extensions.Annotated[
        typing.Optional[OutOfRouteResponseBody], FieldMetadata(alias="outOfRoute"), pydantic.Field(alias="outOfRoute")
    ] = None
    outside_geofence: typing_extensions.Annotated[
        typing.Optional[OutsideGeofenceDataResponseBody],
        FieldMetadata(alias="outsideGeofence"),
        pydantic.Field(alias="outsideGeofence"),
    ] = None
    panic_button: typing_extensions.Annotated[
        typing.Optional[PanicButtonResponseBody],
        FieldMetadata(alias="panicButton"),
        pydantic.Field(alias="panicButton"),
    ] = None
    person_detected: typing_extensions.Annotated[
        typing.Optional[PersonDetectedResponseBody],
        FieldMetadata(alias="personDetected"),
        pydantic.Field(alias="personDetected"),
    ] = None
    reading: typing.Optional[ReadingTriggerDetailsResponseBody] = None
    reefer_temperature: typing_extensions.Annotated[
        typing.Optional[ReeferTemperatureResponseBody],
        FieldMetadata(alias="reeferTemperature"),
        pydantic.Field(alias="reeferTemperature"),
    ] = None
    route_stop_arrival: typing_extensions.Annotated[
        typing.Optional[RouteStopDetailsObjectResponseBody],
        FieldMetadata(alias="routeStopArrival"),
        pydantic.Field(alias="routeStopArrival"),
    ] = None
    route_stop_departure: typing_extensions.Annotated[
        typing.Optional[RouteStopDetailsObjectResponseBody],
        FieldMetadata(alias="routeStopDeparture"),
        pydantic.Field(alias="routeStopDeparture"),
    ] = None
    route_stop_eta: typing_extensions.Annotated[
        typing.Optional[RouteStopEtaResponseBody],
        FieldMetadata(alias="routeStopETA"),
        pydantic.Field(alias="routeStopETA"),
    ] = None
    scheduled_maintenance: typing_extensions.Annotated[
        typing.Optional[ScheduledMaintenanceDataResponseBody],
        FieldMetadata(alias="scheduledMaintenance"),
        pydantic.Field(alias="scheduledMaintenance"),
    ] = None
    scheduled_maintenance_by_engine_hours: typing_extensions.Annotated[
        typing.Optional[ScheduledMaintenanceByEngineHoursResponseBody],
        FieldMetadata(alias="scheduledMaintenanceByEngineHours"),
        pydantic.Field(alias="scheduledMaintenanceByEngineHours"),
    ] = None
    scheduled_maintenance_odometer: typing_extensions.Annotated[
        typing.Optional[ScheduledMaintenanceOdometerDataResponseBody],
        FieldMetadata(alias="scheduledMaintenanceOdometer"),
        pydantic.Field(alias="scheduledMaintenanceOdometer"),
    ] = None
    severe_speeding: typing_extensions.Annotated[
        typing.Optional[SevereSpeedingStartedResponseObjectResponseBody],
        FieldMetadata(alias="severeSpeeding"),
        pydantic.Field(alias="severeSpeeding"),
    ] = None
    site_gateway_disconnected: typing_extensions.Annotated[
        typing.Optional[SiteGatewayDisconnectedResponseBody],
        FieldMetadata(alias="siteGatewayDisconnected"),
        pydantic.Field(alias="siteGatewayDisconnected"),
    ] = None
    speed: typing.Optional[SpeedDataResponseBody] = None
    sudden_fuel_level_drop: typing_extensions.Annotated[
        typing.Optional[SuddenFuelLevelDropResponseBody],
        FieldMetadata(alias="suddenFuelLevelDrop"),
        pydantic.Field(alias="suddenFuelLevelDrop"),
    ] = None
    sudden_fuel_level_rise: typing_extensions.Annotated[
        typing.Optional[SuddenFuelLevelRiseResponseBody],
        FieldMetadata(alias="suddenFuelLevelRise"),
        pydantic.Field(alias="suddenFuelLevelRise"),
    ] = None
    tampering_detected: typing_extensions.Annotated[
        typing.Optional[TamperingDetectedResponseBody],
        FieldMetadata(alias="tamperingDetected"),
        pydantic.Field(alias="tamperingDetected"),
    ] = None
    tire_faults: typing_extensions.Annotated[
        typing.Optional[TireFaultsResponseBody], FieldMetadata(alias="tireFaults"), pydantic.Field(alias="tireFaults")
    ] = None
    unassigned_driving: typing_extensions.Annotated[
        typing.Optional[UnassignedDrivingDataResponseBody],
        FieldMetadata(alias="unassignedDriving"),
        pydantic.Field(alias="unassignedDriving"),
    ] = None
    vehicle_battery_voltage: typing_extensions.Annotated[
        typing.Optional[VehicleBatteryVoltageResponseBody],
        FieldMetadata(alias="vehicleBatteryVoltage"),
        pydantic.Field(alias="vehicleBatteryVoltage"),
    ] = None
    vehicle_def_level_percentage: typing_extensions.Annotated[
        typing.Optional[VehicleDefLevelPercentageResponseBody],
        FieldMetadata(alias="vehicleDefLevelPercentage"),
        pydantic.Field(alias="vehicleDefLevelPercentage"),
    ] = None
    vehicle_detected: typing_extensions.Annotated[
        typing.Optional[VehicleDetectedResponseBody],
        FieldMetadata(alias="vehicleDetected"),
        pydantic.Field(alias="vehicleDetected"),
    ] = None
    vehicle_faults: typing_extensions.Annotated[
        typing.Optional[VehicleFaultsResponseBody],
        FieldMetadata(alias="vehicleFaults"),
        pydantic.Field(alias="vehicleFaults"),
    ] = None
    worker_safety_sos: typing_extensions.Annotated[
        typing.Optional[WorkerSafetySosDataResponseBody],
        FieldMetadata(alias="workerSafetySos"),
        pydantic.Field(alias="workerSafetySos"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .behavior_response_body_coachable_behavior_type import BehaviorResponseBodyCoachableBehaviorType
from .coachable_event_response_body import CoachableEventResponseBody


class BehaviorResponseBody(UniversalBaseModel):
    """
    Object reference for the behavior within the session.
    """

    coachable_behavior_type: typing_extensions.Annotated[
        BehaviorResponseBodyCoachableBehaviorType,
        FieldMetadata(alias="coachableBehaviorType"),
        pydantic.Field(
            alias="coachableBehaviorType",
            description="Coachable behavior type for the behavior in the coaching session.  Valid values: `acceleration`, `braking`, `cameraObstruction`, `crash`, `defensiveDriving`, `didNotYield`, `drinkPolicy`, `drowsy`, `eatingDrinking`, `event`, `falsePositive`, `foodPolicy`, `forwardCollisionWarning`, `genericDistraction`, `harshTurn`, `hosViolation`, `idling`, `laneDeparture`, `lateResponse`, `maskPolicy`, `maxSpeed`, `mobileUsage`, `nearCollison`, `noSeatbelt`, `obstructedCamera`, `outwardObstruction`, `passengerPolicy`, `ranRedLight`, `rollingRailroadCrossing`, `rollingStop`, `rollingStop`, `rollover`, `rolloverProtection`, `rolloverProtectionBrakeControlActivated`, `rolloverProtectionEngineControlActivated`, `severeSpeeding`, `smoking`, `speeding`, `tailgating`, `unknown`, `yawControl`, `yawControlBrakeControlActivated`, `yawControlEngineControlActivated`",
        ),
    ]
    coachable_events: typing_extensions.Annotated[
        typing.Optional[typing.List[CoachableEventResponseBody]],
        FieldMetadata(alias="coachableEvents"),
        pydantic.Field(
            alias="coachableEvents",
            description="Object references for the coachableEvents within the behavior. Returned when includeCoachableEvents is 'true'. Capped at 100 coachable events per Coaching session. For sessions where coachable events exceed 100, please visit the Samsara dashboard to address this coaching session. Corresponds to the following unique identifiers: Non-Speeding Safety events - unique Samsara ID of the safety event as 'vehicleId - eventMS'. Speeding events - unique UUID of the event. Day of HOS Violations - unique Samsara ID of the day of violations as '{driverId},{date}'. Idling events - unique UUID of the event.",
        ),
    ] = None
    id: str = pydantic.Field()
    """
    Unique ID for the coaching behavior.
    """

    last_coached_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="lastCoachedTime"),
        pydantic.Field(alias="lastCoachedTime", description="Time of last coached date for the same behavior label."),
    ]
    note: typing.Optional[str] = pydantic.Field(default=None)
    """
    Associated note for the coaching behavior. Returned when present.
    """

    updated_at_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="updatedAtTime"),
        pydantic.Field(alias="updatedAtTime", description="Time of coaching behavior update in UTC."),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



import json
import os
from typing import Dict, Any, Optional, Protocol


class SSMClientProtocol(Protocol):
    """Protocol defining the SSM client interface we need."""

    def get_parameter(self, Name: str, WithDecryption: bool) -> Dict[str, Any]:
        """Get parameter from SSM."""
        ...


class Secrets:
    """
    Handles loading secrets from either local environment or AWS Parameter Store.
    """

    def __init__(self, client: Optional[SSMClientProtocol]) -> None:
        """
        Initialize the Secrets handler.

        Args:
            client: SSM client for accessing AWS Parameter Store, or None for local development
        """
        self.__client = client

    def load(self) -> Dict[str, Any]:
        """
        Load secrets from the appropriate source based on environment.

        When running locally, secrets are loaded from the
        'SamsaraFunctionLocalSecretsJson' environment variable.

        Returns:
            Dict containing the parsed secrets
        """
        if self.__client is None:
            return json.loads(os.environ.get("SamsaraFunctionLocalSecretsJson", "{}"))

        res = self.__client.get_parameter(
            Name=os.environ["SamsaraFunctionSecretsPath"], WithDecryption=True
        )

        value = res["Parameter"]["Value"]
        if value == "null":
            return {}

        return json.loads(value)


class Function:
    """
    Handles environment detection and secrets management.
    """

    def __init__(
        self, is_local: bool = os.environ.get("AWS_EXECUTION_ENV", "") == ""
    ) -> None:
        """
        Initialize the function with appropriate configuration based on environment.

        Args:
            is_local: Flag indicating if running in local development environment

        Note:
            The is_local parameter doesn't need to be set explicitly as it defaults to
            checking if AWS_EXECUTION_ENV is empty. This environment variable is automatically
            set by AWS Lambda when running in the cloud, but will be empty during local
            development, making the detection automatic in most cases.
        """
        if is_local:
            self.__secrets = Secrets(None)
        else:
            # boto3 is always available in the execution enviroment and not required in development.
            import boto3  # type: ignore

            sts = boto3.client("sts")
            res = sts.assume_role(
                RoleArn=os.environ["SamsaraFunctionExecRoleArn"],
                RoleSessionName=os.environ["SamsaraFunctionName"],
            )

            credentials = {
                "aws_access_key_id": res["Credentials"]["AccessKeyId"],
                "aws_secret_access_key": res["Credentials"]["SecretAccessKey"],
                "aws_session_token": res["Credentials"]["SessionToken"],
            }

            self.__secrets = Secrets(boto3.client("ssm", **credentials))

    def secrets(self) -> Secrets:
        """
        Get the secrets handler.

        Returns:
            The configured Secrets instance
        """
        return self.__secrets


# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .alert_object_asset_response_body import AlertObjectAssetResponseBody
from .continuous_reading_alert_threshold_response_body import ContinuousReadingAlertThresholdResponseBody
from .enum_reading_alert_threshold_response_body import EnumReadingAlertThresholdResponseBody


class ReadingTriggerDetailsResponseBody(UniversalBaseModel):
    """
    Details specific to Reading Triggers.
    """

    asset: typing.Optional[AlertObjectAssetResponseBody] = None
    continuous_threshold: typing_extensions.Annotated[
        typing.Optional[ContinuousReadingAlertThresholdResponseBody],
        FieldMetadata(alias="continuousThreshold"),
        pydantic.Field(alias="continuousThreshold"),
    ] = None
    enum_threshold: typing_extensions.Annotated[
        typing.Optional[EnumReadingAlertThresholdResponseBody],
        FieldMetadata(alias="enumThreshold"),
        pydantic.Field(alias="enumThreshold"),
    ] = None
    id: str = pydantic.Field()
    """
    The ID of the reading.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

DvirGetDvirResponseBodySafetyStatus = typing.Union[typing.Literal["unknown", "safe", "unsafe", "resolved"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

import typing

V1AssetsReeferReeferStatsPowerStatusStatus = typing.Union[
    typing.Literal["Off", "Active", "Active (Start/Stop)", "Active (Continuous)"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

FormsFieldDefinitionObjectResponseBodyAllowedDateTimeValueType = typing.Union[
    typing.Literal["datetime", "date", "time"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .aemp_equipment_with_additional_fields_response_body import AempEquipmentWithAdditionalFieldsResponseBody
from .aemp_link_response_body import AempLinkResponseBody


class AempFleetListResponseBody(UniversalBaseModel):
    """
    Contains a list of equipment objects and links
    """

    equipment: typing_extensions.Annotated[
        typing.List[AempEquipmentWithAdditionalFieldsResponseBody],
        FieldMetadata(alias="Equipment"),
        pydantic.Field(alias="Equipment", description="The list of Equipment objects."),
    ]
    links: typing_extensions.Annotated[
        typing.List[AempLinkResponseBody],
        FieldMetadata(alias="Links"),
        pydantic.Field(alias="Links", description="The list of links associated with the current API request."),
    ]
    snapshot_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="snapshotTime"),
        pydantic.Field(
            alias="snapshotTime",
            description="Date and time at which the snapshot of the fleet was created in RFC 3339 format.",
        ),
    ]
    version: str = pydantic.Field()
    """
    The version of the ISO/TS 15143-3 standard
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .v_1_vehicle_maintenance_j_1939_check_engine_light import V1VehicleMaintenanceJ1939CheckEngineLight
from .v_1_vehicle_maintenance_j_1939_diagnostic_trouble_codes import V1VehicleMaintenanceJ1939DiagnosticTroubleCodes


class V1VehicleMaintenanceJ1939(UniversalBaseModel):
    """
    J1939 based data. Null if no data is available.
    """

    check_engine_light: typing_extensions.Annotated[
        typing.Optional[V1VehicleMaintenanceJ1939CheckEngineLight],
        FieldMetadata(alias="checkEngineLight"),
        pydantic.Field(alias="checkEngineLight"),
    ] = None
    diagnostic_trouble_codes: typing_extensions.Annotated[
        typing.Optional[typing.List[V1VehicleMaintenanceJ1939DiagnosticTroubleCodes]],
        FieldMetadata(alias="diagnosticTroubleCodes"),
        pydantic.Field(alias="diagnosticTroubleCodes", description="J1939 DTCs."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class CreateUserRequestRoles(UniversalBaseModel):
    """
    A role that applies to a user. If the role has a `tagId`, then the role applies for that tag. If there is no `tagId`, then the role applies at the organizational level. A user may have many tag-specific roles, but may only have one organizational role. If the organizational level role has higher privileges than a tag-specific role, then the organizational role privileges will take precedence.
    """

    role_id: typing_extensions.Annotated[
        str, FieldMetadata(alias="roleId"), pydantic.Field(alias="roleId", description="The unique ID for the role.")
    ]
    tag_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="tagId"),
        pydantic.Field(alias="tagId", description="ID of the tag this role applies to."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .safety_event_behavior_label_name import SafetyEventBehaviorLabelName
from .safety_event_behavior_label_source import SafetyEventBehaviorLabelSource
from .safety_event_behavior_label_type import SafetyEventBehaviorLabelType


class SafetyEventBehaviorLabel(UniversalBaseModel):
    """
    The label and source of the label associated with the safety event.
    """

    label: SafetyEventBehaviorLabelType
    name: typing.Optional[SafetyEventBehaviorLabelName] = None
    source: SafetyEventBehaviorLabelSource

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

FormSubmissionResponseObjectResponseBodyStatus = typing.Union[
    typing.Literal["notStarted", "completed", "archived", "inProgress", "needsReview", "changesRequested", "approved"],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .vehicle_stats_fault_codes_value_j_1939_vendor_specific_fields import (
    VehicleStatsFaultCodesValueJ1939VendorSpecificFields,
)


class VehicleStatsFaultCodesValueJ1939DiagnosticTroubleCodes(UniversalBaseModel):
    """
    Diagnostic trouble code for J1939 vehicles.
    """

    fmi_description: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="fmiDescription"),
        pydantic.Field(alias="fmiDescription", description="A short description of the FMI identifier, if available."),
    ] = None
    fmi_id: typing_extensions.Annotated[
        int, FieldMetadata(alias="fmiId"), pydantic.Field(alias="fmiId", description="The FMI identifier.")
    ]
    mil_status: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="milStatus"),
        pydantic.Field(alias="milStatus", description="The MIL status, indicating a check engine light."),
    ]
    occurrence_count: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="occurrenceCount"),
        pydantic.Field(alias="occurrenceCount", description="The number of times this fault has triggered."),
    ]
    source_address_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="sourceAddressName"),
        pydantic.Field(alias="sourceAddressName", description="The source address name corresponding to the txId"),
    ] = None
    spn_description: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="spnDescription"),
        pydantic.Field(alias="spnDescription", description="A short description of the SPN identifier, if available."),
    ] = None
    spn_id: typing_extensions.Annotated[
        int, FieldMetadata(alias="spnId"), pydantic.Field(alias="spnId", description="The SPN identifier.")
    ]
    tx_id: typing_extensions.Annotated[
        int, FieldMetadata(alias="txId"), pydantic.Field(alias="txId", description="The TX identifier.")
    ]
    vendor_specific_fields: typing_extensions.Annotated[
        typing.Optional[VehicleStatsFaultCodesValueJ1939VendorSpecificFields],
        FieldMetadata(alias="vendorSpecificFields"),
        pydantic.Field(alias="vendorSpecificFields"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class EngineOffDetailsObjectRequestBody(UniversalBaseModel):
    """
    Details specific to Engine Off
    """

    min_duration_milliseconds: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="minDurationMilliseconds"),
        pydantic.Field(
            alias="minDurationMilliseconds",
            description="The number of milliseconds the trigger needs to stay active before alerting.",
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class DistanceTraveledObjectResponseBody(UniversalBaseModel):
    """
    The distance traveled information of the log.
    """

    drive_distance_meters: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="driveDistanceMeters"),
        pydantic.Field(
            alias="driveDistanceMeters", description="Distance driven in meters, rounded to two decimal places."
        ),
    ] = None
    personal_conveyance_distance_meters: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="personalConveyanceDistanceMeters"),
        pydantic.Field(
            alias="personalConveyanceDistanceMeters",
            description="Distance driven for personal conveyance, rounded to two decimal places.",
        ),
    ] = None
    yard_move_distance_meters: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="yardMoveDistanceMeters"),
        pydantic.Field(
            alias="yardMoveDistanceMeters", description="Distance driven for yard moves, rounded to two decimal places."
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .equipment_engine_rpm import EquipmentEngineRpm
from .equipment_engine_seconds import EquipmentEngineSeconds
from .equipment_engine_state import EquipmentEngineState
from .equipment_engine_total_idle_time_minutes import EquipmentEngineTotalIdleTimeMinutes
from .equipment_fuel_percent import EquipmentFuelPercent
from .equipment_gateway_engine_seconds import EquipmentGatewayEngineSeconds
from .equipment_gateway_engine_state import EquipmentGatewayEngineState
from .equipment_gps_odometer_meters import EquipmentGpsOdometerMeters
from .equipment_id import EquipmentId
from .equipment_name import EquipmentName
from .equipment_obd_engine_seconds import EquipmentObdEngineSeconds
from .equipment_obd_engine_state import EquipmentObdEngineState
from .equipment_stats_gps import EquipmentStatsGps


class EquipmentStatsResponseData(UniversalBaseModel):
    """
    A unit of equipment and its most recent stat.
    """

    engine_rpm: typing_extensions.Annotated[
        typing.Optional[EquipmentEngineRpm], FieldMetadata(alias="engineRpm"), pydantic.Field(alias="engineRpm")
    ] = None
    engine_seconds: typing_extensions.Annotated[
        typing.Optional[EquipmentEngineSeconds],
        FieldMetadata(alias="engineSeconds"),
        pydantic.Field(alias="engineSeconds"),
    ] = None
    engine_state: typing_extensions.Annotated[
        typing.Optional[EquipmentEngineState], FieldMetadata(alias="engineState"), pydantic.Field(alias="engineState")
    ] = None
    engine_total_idle_time_minutes: typing_extensions.Annotated[
        typing.Optional[EquipmentEngineTotalIdleTimeMinutes],
        FieldMetadata(alias="engineTotalIdleTimeMinutes"),
        pydantic.Field(alias="engineTotalIdleTimeMinutes"),
    ] = None
    fuel_percent: typing_extensions.Annotated[
        typing.Optional[EquipmentFuelPercent], FieldMetadata(alias="fuelPercent"), pydantic.Field(alias="fuelPercent")
    ] = None
    gateway_engine_seconds: typing_extensions.Annotated[
        typing.Optional[EquipmentGatewayEngineSeconds],
        FieldMetadata(alias="gatewayEngineSeconds"),
        pydantic.Field(alias="gatewayEngineSeconds"),
    ] = None
    gateway_engine_state: typing_extensions.Annotated[
        typing.Optional[EquipmentGatewayEngineState],
        FieldMetadata(alias="gatewayEngineState"),
        pydantic.Field(alias="gatewayEngineState"),
    ] = None
    gps: typing.Optional[EquipmentStatsGps] = None
    gps_odometer_meters: typing_extensions.Annotated[
        typing.Optional[EquipmentGpsOdometerMeters],
        FieldMetadata(alias="gpsOdometerMeters"),
        pydantic.Field(alias="gpsOdometerMeters"),
    ] = None
    id: EquipmentId
    name: EquipmentName
    obd_engine_seconds: typing_extensions.Annotated[
        typing.Optional[EquipmentObdEngineSeconds],
        FieldMetadata(alias="obdEngineSeconds"),
        pydantic.Field(alias="obdEngineSeconds"),
    ] = None
    obd_engine_state: typing_extensions.Annotated[
        typing.Optional[EquipmentObdEngineState],
        FieldMetadata(alias="obdEngineState"),
        pydantic.Field(alias="obdEngineState"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .safety_event_v_2_media_response_body_camera_role import SafetyEventV2MediaResponseBodyCameraRole
from .safety_event_v_2_media_response_body_input import SafetyEventV2MediaResponseBodyInput


class SafetyEventV2MediaResponseBody(UniversalBaseModel):
    """
    Media asset available for the safety event.
    """

    auxcam_serial: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="auxcamSerial"),
        pydantic.Field(alias="auxcamSerial", description="The serial number of the auxiliary camera device."),
    ] = None
    camera_role: typing_extensions.Annotated[
        typing.Optional[SafetyEventV2MediaResponseBodyCameraRole],
        FieldMetadata(alias="cameraRole"),
        pydantic.Field(
            alias="cameraRole",
            description="The currently assigned role name of the camera. May change if camera role settings are updated.  Valid values: `leftMirrorMount`, `leftSide`, `rightMirrorMount`, `rightSide`, `rearHigh`, `rearBumper`, `inCab`, `front`, `hopper`, `other1`, `other2`, `other3`, `other4`, `leftBev`, `rightBev`, `rearBev`, `frontBev`, `otherBev`, `bevNotUsed`",
        ),
    ] = None
    input: SafetyEventV2MediaResponseBodyInput = pydantic.Field()
    """
    The media input type of the camera.  Valid values: `dashcamRoadFacing`, `dashcamDriverFacing`, `analog1`, `analog2`, `analog3`, `analog4`
    """

    url: str = pydantic.Field()
    """
    URL to download the media asset.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

DetectionLogDetectionObjectResponseBodyTriggerDetectionLabel = typing.Union[
    typing.Literal[
        "acceleration",
        "braking",
        "crash",
        "drowsy",
        "eatingDrinking",
        "edgeRailroadCrossingViolation",
        "followingDistance",
        "forwardCollisionWarning",
        "genericDistraction",
        "harshTurn",
        "heavySpeeding",
        "laneDeparture",
        "lightSpeeding",
        "maxSpeed",
        "mobileUsage",
        "moderateSpeeding",
        "noSeatbelt",
        "obstructedCamera",
        "passenger",
        "policyViolationMask",
        "ranRedLight",
        "rearCollisionWarning",
        "rollingStop",
        "rolloverProtection",
        "severeSpeeding",
        "smoking",
        "speeding",
        "unsafeParking",
        "vehicleInBlindSpotWarning",
        "vulnerableRoadUserCollisionWarning",
        "yawControl",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class TrailerStatReeferSupplyAirTemperatureMilliCZone2TypeResponseBody(UniversalBaseModel):
    """
    Supply or discharge air temperature of zone 2 of the reefer. This is the temperature of the air as it leaves the cooling unit.
    """

    time: str = pydantic.Field()
    """
    UTC timestamp in RFC 3339 format.
    """

    value: int = pydantic.Field()
    """
    The supply or discharge air temperature reading in millidegree Celsius.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .recipient_object_response_body_notification_types_item import RecipientObjectResponseBodyNotificationTypesItem
from .recipient_object_response_body_type import RecipientObjectResponseBodyType


class RecipientObjectResponseBody(UniversalBaseModel):
    """
    Recipient of an Action. One of userId contactId or roleId needs to be set.
    """

    contact_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="contactId"),
        pydantic.Field(alias="contactId", description="The ID of the contact."),
    ] = None
    notification_types: typing_extensions.Annotated[
        typing.Optional[typing.List[RecipientObjectResponseBodyNotificationTypesItem]],
        FieldMetadata(alias="notificationTypes"),
        pydantic.Field(alias="notificationTypes", description="How the user/contact/role should be notified."),
    ] = None
    role_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="roleId"),
        pydantic.Field(alias="roleId", description="The ID of the role."),
    ] = None
    type: RecipientObjectResponseBodyType = pydantic.Field()
    """
    The type of recipients  Valid values: `user`, `contact`, `role`
    """

    user_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="userId"),
        pydantic.Field(alias="userId", description="The ID of the user."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .attribute_tiny import AttributeTiny
from .driver_activation_status import DriverActivationStatus
from .driver_carrier_settings import DriverCarrierSettings
from .driver_created_at_time import DriverCreatedAtTime
from .driver_eld_adverse_weather_exemption_enabled import DriverEldAdverseWeatherExemptionEnabled
from .driver_eld_big_day_exemption_enabled import DriverEldBigDayExemptionEnabled
from .driver_eld_day_start_hour import DriverEldDayStartHour
from .driver_eld_exempt import DriverEldExempt
from .driver_eld_exempt_reason import DriverEldExemptReason
from .driver_eld_pc_enabled import DriverEldPcEnabled
from .driver_eld_settings import DriverEldSettings
from .driver_eld_ym_enabled import DriverEldYmEnabled
from .driver_external_ids import DriverExternalIds
from .driver_has_driving_features_hidden import DriverHasDrivingFeaturesHidden
from .driver_has_vehicle_unpinning_enabled import DriverHasVehicleUnpinningEnabled
from .driver_id import DriverId
from .driver_id_card_code import DriverIdCardCode
from .driver_is_deactivated import DriverIsDeactivated
from .driver_license_number import DriverLicenseNumber
from .driver_license_state import DriverLicenseState
from .driver_locale import DriverLocale
from .driver_name import DriverName
from .driver_notes import DriverNotes
from .driver_phone import DriverPhone
from .driver_profile_image_url import DriverProfileImageUrl
from .driver_static_assigned_vehicle import DriverStaticAssignedVehicle
from .driver_tachograph_card_number import DriverTachographCardNumber
from .driver_tags import DriverTags
from .driver_timezone import DriverTimezone
from .driver_updated_at_time import DriverUpdatedAtTime
from .driver_username import DriverUsername
from .driver_vehicle_group_tag import DriverVehicleGroupTag
from .driver_waiting_time_duty_status_enabled import DriverWaitingTimeDutyStatusEnabled
from .tag_tiny_response import TagTinyResponse
from .us_driver_ruleset_override import UsDriverRulesetOverride


class Driver(UniversalBaseModel):
    """
    A driver object
    """

    attributes: typing.Optional[typing.List[AttributeTiny]] = pydantic.Field(default=None)
    """
    A minified attribute
    """

    carrier_settings: typing_extensions.Annotated[
        typing.Optional[DriverCarrierSettings],
        FieldMetadata(alias="carrierSettings"),
        pydantic.Field(alias="carrierSettings"),
    ] = None
    created_at_time: typing_extensions.Annotated[
        typing.Optional[DriverCreatedAtTime],
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(alias="createdAtTime"),
    ] = None
    current_id_card_code: typing_extensions.Annotated[
        typing.Optional[DriverIdCardCode],
        FieldMetadata(alias="currentIdCardCode"),
        pydantic.Field(alias="currentIdCardCode"),
    ] = None
    driver_activation_status: typing_extensions.Annotated[
        typing.Optional[DriverActivationStatus],
        FieldMetadata(alias="driverActivationStatus"),
        pydantic.Field(alias="driverActivationStatus"),
    ] = None
    eld_adverse_weather_exemption_enabled: typing_extensions.Annotated[
        typing.Optional[DriverEldAdverseWeatherExemptionEnabled],
        FieldMetadata(alias="eldAdverseWeatherExemptionEnabled"),
        pydantic.Field(alias="eldAdverseWeatherExemptionEnabled"),
    ] = None
    eld_big_day_exemption_enabled: typing_extensions.Annotated[
        typing.Optional[DriverEldBigDayExemptionEnabled],
        FieldMetadata(alias="eldBigDayExemptionEnabled"),
        pydantic.Field(alias="eldBigDayExemptionEnabled"),
    ] = None
    eld_day_start_hour: typing_extensions.Annotated[
        typing.Optional[DriverEldDayStartHour],
        FieldMetadata(alias="eldDayStartHour"),
        pydantic.Field(alias="eldDayStartHour"),
    ] = None
    eld_exempt: typing_extensions.Annotated[
        typing.Optional[DriverEldExempt], FieldMetadata(alias="eldExempt"), pydantic.Field(alias="eldExempt")
    ] = None
    eld_exempt_reason: typing_extensions.Annotated[
        typing.Optional[DriverEldExemptReason],
        FieldMetadata(alias="eldExemptReason"),
        pydantic.Field(alias="eldExemptReason"),
    ] = None
    eld_pc_enabled: typing_extensions.Annotated[
        typing.Optional[DriverEldPcEnabled], FieldMetadata(alias="eldPcEnabled"), pydantic.Field(alias="eldPcEnabled")
    ] = None
    eld_settings: typing_extensions.Annotated[
        typing.Optional[DriverEldSettings], FieldMetadata(alias="eldSettings"), pydantic.Field(alias="eldSettings")
    ] = None
    eld_ym_enabled: typing_extensions.Annotated[
        typing.Optional[DriverEldYmEnabled], FieldMetadata(alias="eldYmEnabled"), pydantic.Field(alias="eldYmEnabled")
    ] = None
    external_ids: typing_extensions.Annotated[
        typing.Optional[DriverExternalIds], FieldMetadata(alias="externalIds"), pydantic.Field(alias="externalIds")
    ] = None
    has_driving_features_hidden: typing_extensions.Annotated[
        typing.Optional[DriverHasDrivingFeaturesHidden],
        FieldMetadata(alias="hasDrivingFeaturesHidden"),
        pydantic.Field(alias="hasDrivingFeaturesHidden"),
    ] = None
    has_vehicle_unpinning_enabled: typing_extensions.Annotated[
        typing.Optional[DriverHasVehicleUnpinningEnabled],
        FieldMetadata(alias="hasVehicleUnpinningEnabled"),
        pydantic.Field(alias="hasVehicleUnpinningEnabled"),
    ] = None
    id: typing.Optional[DriverId] = None
    is_deactivated: typing_extensions.Annotated[
        typing.Optional[DriverIsDeactivated],
        FieldMetadata(alias="isDeactivated"),
        pydantic.Field(alias="isDeactivated"),
    ] = None
    license_number: typing_extensions.Annotated[
        typing.Optional[DriverLicenseNumber],
        FieldMetadata(alias="licenseNumber"),
        pydantic.Field(alias="licenseNumber"),
    ] = None
    license_state: typing_extensions.Annotated[
        typing.Optional[DriverLicenseState], FieldMetadata(alias="licenseState"), pydantic.Field(alias="licenseState")
    ] = None
    locale: typing.Optional[DriverLocale] = None
    name: typing.Optional[DriverName] = None
    notes: typing.Optional[DriverNotes] = None
    peer_group_tag: typing_extensions.Annotated[
        typing.Optional[TagTinyResponse], FieldMetadata(alias="peerGroupTag"), pydantic.Field(alias="peerGroupTag")
    ] = None
    phone: typing.Optional[DriverPhone] = None
    profile_image_url: typing_extensions.Annotated[
        typing.Optional[DriverProfileImageUrl],
        FieldMetadata(alias="profileImageUrl"),
        pydantic.Field(alias="profileImageUrl"),
    ] = None
    static_assigned_vehicle: typing_extensions.Annotated[
        typing.Optional[DriverStaticAssignedVehicle],
        FieldMetadata(alias="staticAssignedVehicle"),
        pydantic.Field(alias="staticAssignedVehicle"),
    ] = None
    tachograph_card_number: typing_extensions.Annotated[
        typing.Optional[DriverTachographCardNumber],
        FieldMetadata(alias="tachographCardNumber"),
        pydantic.Field(alias="tachographCardNumber"),
    ] = None
    tags: typing.Optional[DriverTags] = None
    timezone: typing.Optional[DriverTimezone] = None
    updated_at_time: typing_extensions.Annotated[
        typing.Optional[DriverUpdatedAtTime],
        FieldMetadata(alias="updatedAtTime"),
        pydantic.Field(alias="updatedAtTime"),
    ] = None
    us_driver_ruleset_override: typing_extensions.Annotated[
        typing.Optional[UsDriverRulesetOverride],
        FieldMetadata(alias="usDriverRulesetOverride"),
        pydantic.Field(alias="usDriverRulesetOverride"),
    ] = None
    username: typing.Optional[DriverUsername] = None
    vehicle_group_tag: typing_extensions.Annotated[
        typing.Optional[DriverVehicleGroupTag],
        FieldMetadata(alias="vehicleGroupTag"),
        pydantic.Field(alias="vehicleGroupTag"),
    ] = None
    waiting_time_duty_status_enabled: typing_extensions.Annotated[
        typing.Optional[DriverWaitingTimeDutyStatusEnabled],
        FieldMetadata(alias="waitingTimeDutyStatusEnabled"),
        pydantic.Field(alias="waitingTimeDutyStatusEnabled"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

SafetyEventActivityFeedItemResponseBodyType = typing.Union[
    typing.Literal["BehaviorLabelActivityType", "CoachingStateActivityType", "CreateSafetyEventActivityType"],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.carrier_proposed_assignment_response import CarrierProposedAssignmentResponse
from ..types.list_carrier_proposed_assignment_response import ListCarrierProposedAssignmentResponse
from .raw_client import AsyncRawCarrierProposedAssignmentsClient, RawCarrierProposedAssignmentsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CarrierProposedAssignmentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCarrierProposedAssignmentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCarrierProposedAssignmentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCarrierProposedAssignmentsClient
        """
        return self._raw_client

    def list_carrier_proposed_assignments(
        self,
        *,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        active_time: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListCarrierProposedAssignmentResponse:
        """
        Show the assignments created by the POST fleet/carrier-proposed-assignments. This endpoint will only show the assignments that are active for drivers and currently visible to them in the driver app. Once a proposed assignment has been accepted, the endpoint will not return any data.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Carrier-Proposed Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of driver IDs and externalIds. Example: `driverIds=1234,5678,payroll:4841`

        active_time : typing.Optional[str]
            If specified, shows assignments that will be active at this time. Defaults to now, which would show current active assignments. In RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListCarrierProposedAssignmentResponse
            Returns the assignments that drivers would see in the future, if any.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.carrier_proposed_assignments.list_carrier_proposed_assignments()
        """
        _response = self._raw_client.list_carrier_proposed_assignments(
            limit=limit, after=after, driver_ids=driver_ids, active_time=active_time, request_options=request_options
        )
        return _response.data

    def create_carrier_proposed_assignment(
        self,
        *,
        driver_id: str,
        vehicle_id: str,
        active_time: typing.Optional[str] = OMIT,
        shipping_docs: typing.Optional[str] = OMIT,
        trailer_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        trailer_names: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CarrierProposedAssignmentResponse:
        """
        Creates a new assignment that a driver can later use. Each driver can only have one future assignment.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Carrier-Proposed Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        driver_id : str
            ID for the driver for the driver that this assignment is for. This can be either a unique Samsara ID or an external ID for the driver.

        vehicle_id : str
            ID for the vehicle to propose. This can be either a unique Samsara ID or an external ID for the vehicle.

        active_time : typing.Optional[str]
            Time after which this assignment will be active and visible to the driver on the mobile app. Not setting it makes it active now. UTC timestamp in RFC 3339 format. Example: `2020-01-27T07:06:25Z`.

        shipping_docs : typing.Optional[str]
            Shipping Documents that this assignment will propose to the driver.

        trailer_ids : typing.Optional[typing.Sequence[str]]
            IDs of trailers to propose. This can be either a unique Samsara IDs or an external IDs for the trailers. (forbidden if trailerNames is set)

        trailer_names : typing.Optional[typing.Sequence[str]]
            Names of trailers to propose. (forbidden if trailerIds is set)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CarrierProposedAssignmentResponse
            Return the created assignment

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.carrier_proposed_assignments.create_carrier_proposed_assignment(
            driver_id="42",
            vehicle_id="123",
        )
        """
        _response = self._raw_client.create_carrier_proposed_assignment(
            driver_id=driver_id,
            vehicle_id=vehicle_id,
            active_time=active_time,
            shipping_docs=shipping_docs,
            trailer_ids=trailer_ids,
            trailer_names=trailer_names,
            request_options=request_options,
        )
        return _response.data


class AsyncCarrierProposedAssignmentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCarrierProposedAssignmentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCarrierProposedAssignmentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCarrierProposedAssignmentsClient
        """
        return self._raw_client

    async def list_carrier_proposed_assignments(
        self,
        *,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        active_time: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListCarrierProposedAssignmentResponse:
        """
        Show the assignments created by the POST fleet/carrier-proposed-assignments. This endpoint will only show the assignments that are active for drivers and currently visible to them in the driver app. Once a proposed assignment has been accepted, the endpoint will not return any data.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Carrier-Proposed Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of driver IDs and externalIds. Example: `driverIds=1234,5678,payroll:4841`

        active_time : typing.Optional[str]
            If specified, shows assignments that will be active at this time. Defaults to now, which would show current active assignments. In RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListCarrierProposedAssignmentResponse
            Returns the assignments that drivers would see in the future, if any.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.carrier_proposed_assignments.list_carrier_proposed_assignments()


        asyncio.run(main())
        """
        _response = await self._raw_client.list_carrier_proposed_assignments(
            limit=limit, after=after, driver_ids=driver_ids, active_time=active_time, request_options=request_options
        )
        return _response.data

    async def create_carrier_proposed_assignment(
        self,
        *,
        driver_id: str,
        vehicle_id: str,
        active_time: typing.Optional[str] = OMIT,
        shipping_docs: typing.Optional[str] = OMIT,
        trailer_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        trailer_names: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CarrierProposedAssignmentResponse:
        """
        Creates a new assignment that a driver can later use. Each driver can only have one future assignment.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Carrier-Proposed Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        driver_id : str
            ID for the driver for the driver that this assignment is for. This can be either a unique Samsara ID or an external ID for the driver.

        vehicle_id : str
            ID for the vehicle to propose. This can be either a unique Samsara ID or an external ID for the vehicle.

        active_time : typing.Optional[str]
            Time after which this assignment will be active and visible to the driver on the mobile app. Not setting it makes it active now. UTC timestamp in RFC 3339 format. Example: `2020-01-27T07:06:25Z`.

        shipping_docs : typing.Optional[str]
            Shipping Documents that this assignment will propose to the driver.

        trailer_ids : typing.Optional[typing.Sequence[str]]
            IDs of trailers to propose. This can be either a unique Samsara IDs or an external IDs for the trailers. (forbidden if trailerNames is set)

        trailer_names : typing.Optional[typing.Sequence[str]]
            Names of trailers to propose. (forbidden if trailerIds is set)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CarrierProposedAssignmentResponse
            Return the created assignment

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.carrier_proposed_assignments.create_carrier_proposed_assignment(
                driver_id="42",
                vehicle_id="123",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_carrier_proposed_assignment(
            driver_id=driver_id,
            vehicle_id=vehicle_id,
            active_time=active_time,
            shipping_docs=shipping_docs,
            trailer_ids=trailer_ids,
            trailer_names=trailer_names,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

UpdateVehicleRequestAuxInputType12 = typing.Union[
    typing.Literal[
        "none",
        "emergencyLights",
        "emergencyAlarm",
        "stopPaddle",
        "powerTakeOff",
        "plow",
        "sweeper",
        "salter",
        "reefer",
        "door",
        "boom",
        "auxiliaryEngine",
        "generator",
        "eightWayLights",
        "panicButton",
        "privacyButton",
        "frontAxleDrive",
        "weightSensor",
        "other",
        "secondaryFuelSource",
        "ecuPowerTakeOff",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.inline_response_2001 import InlineResponse2001
from .raw_client import AsyncRawLegacyClient, RawLegacyClient


class LegacyClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawLegacyClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawLegacyClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawLegacyClient
        """
        return self._raw_client

    def v_1_get_all_assets(self, *, request_options: typing.Optional[RequestOptions] = None) -> InlineResponse2001:
        """
        **Note: This is a legacy endpoint, consider using [this endpoint](https://developers.samsara.com/reference/listassets) instead. The endpoint will continue to function as documented.**

         Fetch all powered and unpowered equipment.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Equipment** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InlineResponse2001
            List of assets.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.legacy.v_1_get_all_assets()
        """
        _response = self._raw_client.v_1_get_all_assets(request_options=request_options)
        return _response.data


class AsyncLegacyClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawLegacyClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawLegacyClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawLegacyClient
        """
        return self._raw_client

    async def v_1_get_all_assets(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> InlineResponse2001:
        """
        **Note: This is a legacy endpoint, consider using [this endpoint](https://developers.samsara.com/reference/listassets) instead. The endpoint will continue to function as documented.**

         Fetch all powered and unpowered equipment.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Equipment** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InlineResponse2001
            List of assets.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.legacy.v_1_get_all_assets()


        asyncio.run(main())
        """
        _response = await self._raw_client.v_1_get_all_assets(request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

VoiceCoachingSettingsObjectResponseBodyEventsToCoachItem = typing.Union[
    typing.Literal["crash", "inCabSpeeding", "maximumSpeed", "seatbeltUnbuckled", "harshDriving"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class DataInputTinyResponse(UniversalBaseModel):
    """
    A minified data input object
    """

    asset_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="assetId"),
        pydantic.Field(alias="assetId", description="Unique identifier for the data input's asset."),
    ] = None
    data_group: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="dataGroup"),
        pydantic.Field(alias="dataGroup", description="Data group for this data input."),
    ] = None
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier for the data input.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of this data input.
    """

    units: typing.Optional[str] = pydantic.Field(default=None)
    """
    Units of data for this data input.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class ScheduledMaintenanceOdometerTriggerDetailsObjectRequestBody(UniversalBaseModel):
    """
    Details specific to Scheduled Maintenance by Odometer
    """

    due_in_meters: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="dueInMeters"),
        pydantic.Field(
            alias="dueInMeters",
            description="Alert when vehicle odometer has this many meters left until maintenance is due.",
        ),
    ]
    schedule_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="scheduleId"),
        pydantic.Field(alias="scheduleId", description="The id of the maintenance schedule."),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .v_1_vehicle_harsh_event_response_location import V1VehicleHarshEventResponseLocation


class V1VehicleHarshEventResponse(UniversalBaseModel):
    """
    Harsh event details for a vehicle
    """

    download_forward_video_url: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="downloadForwardVideoUrl"),
        pydantic.Field(alias="downloadForwardVideoUrl", description="URL for downloading the forward facing video"),
    ] = None
    download_inward_video_url: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="downloadInwardVideoUrl"),
        pydantic.Field(alias="downloadInwardVideoUrl", description="URL for downloading the inward facing video"),
    ] = None
    download_tracked_inward_video_url: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="downloadTrackedInwardVideoUrl"),
        pydantic.Field(
            alias="downloadTrackedInwardVideoUrl", description="URL for downloading the tracked inward facing video"
        ),
    ] = None
    harsh_event_type: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="harshEventType"),
        pydantic.Field(
            alias="harshEventType",
            description="Type of the harsh event. One of: [Crash, Harsh Acceleration, Harsh Braking, Harsh Turn, ROP Engine, ROP Brake, YC Engine, YC Brake, Harsh Event]",
        ),
    ]
    incident_report_url: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="incidentReportUrl"),
        pydantic.Field(alias="incidentReportUrl", description="URL of the associated incident report page"),
    ]
    is_distracted: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="isDistracted"),
        pydantic.Field(
            alias="isDistracted", description="Whether the driver was deemed distracted during this harsh event"
        ),
    ] = None
    location: typing.Optional[V1VehicleHarshEventResponseLocation] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class CircleRequestBody(UniversalBaseModel):
    """
    Information about a circular geofence. This field is only needed if the geofence is a circle.
    """

    latitude: typing.Optional[float] = pydantic.Field(default=None)
    """
    Latitude of the address. Will be geocoded from formattedAddress if not provided.
    """

    longitude: typing.Optional[float] = pydantic.Field(default=None)
    """
    Longitude of the address. Will be geocoded from formattedAddress if not provided.
    """

    name: str = pydantic.Field()
    """
    The name of the cirlce.
    """

    radius_meters: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="radiusMeters"),
        pydantic.Field(alias="radiusMeters", description="The radius of the circular geofence in meters."),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .fuel_cost_object_response_body import FuelCostObjectResponseBody
from .gaseous_fuel_cost_object_response_body import GaseousFuelCostObjectResponseBody
from .idling_event_address_object_response_body import IdlingEventAddressObjectResponseBody
from .idling_event_asset_object_response_body import IdlingEventAssetObjectResponseBody
from .idling_event_object_v_20251023_response_body_pto_state import IdlingEventObjectV20251023ResponseBodyPtoState
from .idling_event_operator_object_response_body import IdlingEventOperatorObjectResponseBody


class IdlingEventObjectV20251023ResponseBody(UniversalBaseModel):
    """
    An idling event object.
    """

    address: typing.Optional[IdlingEventAddressObjectResponseBody] = None
    air_temperature_millicelsius: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="airTemperatureMillicelsius"),
        pydantic.Field(
            alias="airTemperatureMillicelsius",
            description="The air temperature in millicelsius during the idling event. Value is returned only when it is known.",
        ),
    ] = None
    asset: IdlingEventAssetObjectResponseBody
    duration_milliseconds: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="durationMilliseconds"),
        pydantic.Field(alias="durationMilliseconds", description="The duration of the idling event in milliseconds."),
    ]
    event_uuid: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="eventUuid"),
        pydantic.Field(alias="eventUuid", description="Universally unique identifier of the idling event."),
    ]
    fuel_consumed_milliliters: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="fuelConsumedMilliliters"),
        pydantic.Field(
            alias="fuelConsumedMilliliters",
            description="The amount of fuel consumed in milliliters during the idling event.",
        ),
    ]
    fuel_cost: typing_extensions.Annotated[
        FuelCostObjectResponseBody, FieldMetadata(alias="fuelCost"), pydantic.Field(alias="fuelCost")
    ]
    gaseous_fuel_consumed_grams: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="gaseousFuelConsumedGrams"),
        pydantic.Field(
            alias="gaseousFuelConsumedGrams",
            description="The amount of gaseous fuel consumed in grams during the idling event.",
        ),
    ]
    gaseous_fuel_cost: typing_extensions.Annotated[
        GaseousFuelCostObjectResponseBody,
        FieldMetadata(alias="gaseousFuelCost"),
        pydantic.Field(alias="gaseousFuelCost"),
    ]
    latitude: typing.Optional[float] = pydantic.Field(default=None)
    """
    The latitude of the location where the idling event occurred.
    """

    longitude: typing.Optional[float] = pydantic.Field(default=None)
    """
    The longitude of the location where the idling event occurred.
    """

    operator: typing.Optional[IdlingEventOperatorObjectResponseBody] = None
    pto_state: typing_extensions.Annotated[
        IdlingEventObjectV20251023ResponseBodyPtoState,
        FieldMetadata(alias="ptoState"),
        pydantic.Field(
            alias="ptoState",
            description="The PTO (Power Take-Off) state during the idling event.  Valid values: `active, inactive`",
        ),
    ]
    start_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="startTime"),
        pydantic.Field(alias="startTime", description="The start time of the idling event in RFC 3339 format."),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .author_signature_object_response_body import AuthorSignatureObjectResponseBody
from .dvir_get_dvir_response_body_safety_status import DvirGetDvirResponseBodySafetyStatus
from .dvir_get_dvir_response_body_type import DvirGetDvirResponseBodyType
from .trailer_dvir_object_response_body import TrailerDvirObjectResponseBody
from .vehicle_dvir_object_response_body import VehicleDvirObjectResponseBody
from .walkaround_photo_object_response_body import WalkaroundPhotoObjectResponseBody


class DvirGetDvirResponseBody(UniversalBaseModel):
    author_signature: typing_extensions.Annotated[
        AuthorSignatureObjectResponseBody,
        FieldMetadata(alias="authorSignature"),
        pydantic.Field(alias="authorSignature"),
    ]
    defect_ids: typing_extensions.Annotated[
        typing.Optional[typing.List[str]],
        FieldMetadata(alias="defectIds"),
        pydantic.Field(alias="defectIds", description="IDs of defects registered for the DVIR."),
    ] = None
    dvir_submission_begin_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="dvirSubmissionBeginTime"),
        pydantic.Field(
            alias="dvirSubmissionBeginTime",
            description="Time when driver created DVIR. UTC timestamp in RFC 3339 format.",
        ),
    ]
    dvir_submission_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="dvirSubmissionTime"),
        pydantic.Field(
            alias="dvirSubmissionTime",
            description="Time when driver submitted the DVIR. UTC timestamp in RFC 3339 format.",
        ),
    ]
    formatted_address: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="formattedAddress"), pydantic.Field(alias="formattedAddress")
    ] = None
    id: str = pydantic.Field()
    """
    The unique id of the DVIR
    """

    mechanic_notes: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="mechanicNotes"),
        pydantic.Field(alias="mechanicNotes", description="The mechanics notes on the DVIR."),
    ] = None
    odometer_meters: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="odometerMeters"),
        pydantic.Field(alias="odometerMeters", description="The odometer reading in meters."),
    ] = None
    safety_status: typing_extensions.Annotated[
        typing.Optional[DvirGetDvirResponseBodySafetyStatus],
        FieldMetadata(alias="safetyStatus"),
        pydantic.Field(
            alias="safetyStatus",
            description="The condition of vehicle on which DVIR was done.  Valid values: `unknown`, `safe`, `unsafe`, `resolved`",
        ),
    ] = None
    second_signature: typing_extensions.Annotated[
        typing.Optional[AuthorSignatureObjectResponseBody],
        FieldMetadata(alias="secondSignature"),
        pydantic.Field(alias="secondSignature"),
    ] = None
    third_signature: typing_extensions.Annotated[
        typing.Optional[AuthorSignatureObjectResponseBody],
        FieldMetadata(alias="thirdSignature"),
        pydantic.Field(alias="thirdSignature"),
    ] = None
    trailer: typing.Optional[TrailerDvirObjectResponseBody] = None
    type: DvirGetDvirResponseBodyType = pydantic.Field()
    """
    Inspection type of the DVIR.  Valid values: `preTrip`, `postTrip`, `mechanic`, `unspecified`
    """

    updated_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="updatedAtTime"),
        pydantic.Field(
            alias="updatedAtTime", description="Time of any DVIR updates. UTC timestamp in RFC 3339 format."
        ),
    ]
    vehicle: typing.Optional[VehicleDvirObjectResponseBody] = None
    walkaround_photos: typing_extensions.Annotated[
        typing.Optional[typing.List[WalkaroundPhotoObjectResponseBody]],
        FieldMetadata(alias="walkaroundPhotos"),
        pydantic.Field(alias="walkaroundPhotos", description="List of walkaround photos"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

MinimalRouteStopAuditLogsResponseBodyState = typing.Union[
    typing.Literal["unassigned", "scheduled", "en route", "skipped", "arrived", "departed"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

TrainingAssignmentNearDueDateTriggerAssignmentGroupObjectRequestBodyAssignmentGroupType = typing.Union[
    typing.Literal["CATEGORY", "COURSE"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_gateway_error import BadGatewayError
from ..errors.gateway_timeout_error import GatewayTimeoutError
from ..errors.internal_server_error import InternalServerError
from ..errors.method_not_allowed_error import MethodNotAllowedError
from ..errors.not_found_error import NotFoundError
from ..errors.not_implemented_error import NotImplementedError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.media_retrieval_get_media_retrieval_response_body import MediaRetrievalGetMediaRetrievalResponseBody
from ..types.media_retrieval_list_uploaded_media_response_body import MediaRetrievalListUploadedMediaResponseBody
from ..types.media_retrieval_post_media_retrieval_response_body import MediaRetrievalPostMediaRetrievalResponseBody
from .types.list_uploaded_media_request_inputs_item import ListUploadedMediaRequestInputsItem
from .types.list_uploaded_media_request_media_types_item import ListUploadedMediaRequestMediaTypesItem
from .types.list_uploaded_media_request_trigger_reasons_item import ListUploadedMediaRequestTriggerReasonsItem
from .types.media_retrieval_post_media_retrieval_request_body_inputs_item import (
    MediaRetrievalPostMediaRetrievalRequestBodyInputsItem,
)
from .types.media_retrieval_post_media_retrieval_request_body_media_type import (
    MediaRetrievalPostMediaRetrievalRequestBodyMediaType,
)

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawMediaClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_uploaded_media(
        self,
        *,
        vehicle_ids: str,
        start_time: str,
        end_time: str,
        inputs: typing.Optional[
            typing.Union[ListUploadedMediaRequestInputsItem, typing.Sequence[ListUploadedMediaRequestInputsItem]]
        ] = None,
        media_types: typing.Optional[
            typing.Union[
                ListUploadedMediaRequestMediaTypesItem, typing.Sequence[ListUploadedMediaRequestMediaTypesItem]
            ]
        ] = None,
        trigger_reasons: typing.Optional[
            typing.Union[
                ListUploadedMediaRequestTriggerReasonsItem, typing.Sequence[ListUploadedMediaRequestTriggerReasonsItem]
            ]
        ] = None,
        available_after_time: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[MediaRetrievalListUploadedMediaResponseBody]:
        """
        This endpoint returns a list of all uploaded media (video and still images) matching query parameters, with a maximum query range of one day. Additional media can be retrieved with the [Create a media retrieval request](https://developers.samsara.com/reference/postmediaretrieval) endpoint, and they will be included in the list after they are uploaded. Urls provided by this endpoint expire in 8 hours.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Media Retrieval** under the Safety & Cameras category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        vehicle_ids : str
            A filter on the data based on this comma-separated list of vehicle IDs and externalIds. You can specify up to 20 vehicles. Example: `vehicleIds=1234,5678,samsara.vin:1HGBH41JXMN109186`

        start_time : str
            A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : str
            An end time in RFC 3339 format. End time cannot be more than 24 hours after startTime. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        inputs : typing.Optional[typing.Union[ListUploadedMediaRequestInputsItem, typing.Sequence[ListUploadedMediaRequestInputsItem]]]
            An optional list of desired camera inputs for which to return captured media. If empty, media for all available inputs will be returned.

        media_types : typing.Optional[typing.Union[ListUploadedMediaRequestMediaTypesItem, typing.Sequence[ListUploadedMediaRequestMediaTypesItem]]]
            An optional list of desired media types for which to return captured media. If empty, media for all available media types will be returned. Possible options include: image, videoHighRes.

        trigger_reasons : typing.Optional[typing.Union[ListUploadedMediaRequestTriggerReasonsItem, typing.Sequence[ListUploadedMediaRequestTriggerReasonsItem]]]
            An optional list of desired trigger reasons for which to return captured media. If empty, media for all available trigger reasons will be returned. Possible options include: api, panicButton, periodicStill, rfidEvent, safetyEvent, tripEndStill, tripStartStill, videoRetrieval. videoRetrieval represents media captured for a dashboard video retrieval request.

        available_after_time : typing.Optional[str]
            An optional timestamp in RFC 3339 format that can act as a cursor to track which media has previously been retrieved; only media whose availableAtTime comes after this parameter will be returned. Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[MediaRetrievalListUploadedMediaResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "cameras/media",
            method="GET",
            params={
                "vehicleIds": vehicle_ids,
                "inputs": inputs,
                "mediaTypes": media_types,
                "triggerReasons": trigger_reasons,
                "startTime": start_time,
                "endTime": end_time,
                "availableAfterTime": available_after_time,
                "after": after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    MediaRetrievalListUploadedMediaResponseBody,
                    parse_obj_as(
                        type_=MediaRetrievalListUploadedMediaResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_media_retrieval(
        self, *, retrieval_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[MediaRetrievalGetMediaRetrievalResponseBody]:
        """
        This endpoint returns media information corresponding to a retrieval ID. Retrieval IDs are associated to prior [media retrieval requests](https://developers.samsara.com/reference/postmediaretrieval). Urls provided by this endpoint expire in 8 hours.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Media Retrieval** under the Safety & Cameras category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        retrieval_id : str
            Retrieval ID associated with this media capture request. Examples: 2308cec4-82e0-46f1-8b3c-a3592e5cc21e

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[MediaRetrievalGetMediaRetrievalResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "cameras/media/retrieval",
            method="GET",
            params={
                "retrievalId": retrieval_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    MediaRetrievalGetMediaRetrievalResponseBody,
                    parse_obj_as(
                        type_=MediaRetrievalGetMediaRetrievalResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def post_media_retrieval(
        self,
        *,
        end_time: str,
        inputs: typing.Sequence[MediaRetrievalPostMediaRetrievalRequestBodyInputsItem],
        media_type: MediaRetrievalPostMediaRetrievalRequestBodyMediaType,
        start_time: str,
        vehicle_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[MediaRetrievalPostMediaRetrievalResponseBody]:
        """
        This endpoint creates an asynchronous request to upload certain media from a device. The closest available media to the requested timestamp is returned. Images, videos(both high resolution and low resolution) and hyperlapses are supported.Currently, only unblurred media is supported. If a device is offline, the requested media will be uploaded once it comes back online. Quota limits are enforced for media retrievals made through the API. The Create a media retrieval request response includes information about the media retrieval quota remaining for the organization. The media retrieval quota for the organization is reset at the beginning of each month.The quota is expressed using seconds of High Resolution video. 10 still images are equivalent to a 1 second of High Resolution footage.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Media Retrieval** under the Safety & Cameras category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        end_time : str
            An end time in RFC 3339 format. If endTime is the same as startTime, an image will be captured at startTime. Must be 1 second or more after startTime and no more than the maximum allowed duration per video retrieval type. Please refer to our KB articles for more information. (Examples: 2019-06-13T19:08:55Z, 2019-06-13T19:08:55.455Z, OR 2015-09-15T14:00:42-04:00).

        inputs : typing.Sequence[MediaRetrievalPostMediaRetrievalRequestBodyInputsItem]
            A list of desired camera inputs for which to capture media. Only media with valid inputs (e.g. device has that input stream and device was recording at the time) will be uploaded. An empty list is invalid.

        media_type : MediaRetrievalPostMediaRetrievalRequestBodyMediaType
            The desired media type. If a video is requested, endTime must be after startTime. If an image is requested, endTime must be the same as startTime. Must be one of: image, videoHighRes, videoLowRes. Examples: image, videoHighRes, videoLowRes, hyperlapse.  Valid values: `image`, `videoHighRes`, `videoLowRes`, `hyperlapse`

        start_time : str
            A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        vehicle_id : str
            Vehicle ID for which to initiate media capture. Examples: 1234

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[MediaRetrievalPostMediaRetrievalResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "cameras/media/retrieval",
            method="POST",
            json={
                "endTime": end_time,
                "inputs": inputs,
                "mediaType": media_type,
                "startTime": start_time,
                "vehicleId": vehicle_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    MediaRetrievalPostMediaRetrievalResponseBody,
                    parse_obj_as(
                        type_=MediaRetrievalPostMediaRetrievalResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawMediaClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_uploaded_media(
        self,
        *,
        vehicle_ids: str,
        start_time: str,
        end_time: str,
        inputs: typing.Optional[
            typing.Union[ListUploadedMediaRequestInputsItem, typing.Sequence[ListUploadedMediaRequestInputsItem]]
        ] = None,
        media_types: typing.Optional[
            typing.Union[
                ListUploadedMediaRequestMediaTypesItem, typing.Sequence[ListUploadedMediaRequestMediaTypesItem]
            ]
        ] = None,
        trigger_reasons: typing.Optional[
            typing.Union[
                ListUploadedMediaRequestTriggerReasonsItem, typing.Sequence[ListUploadedMediaRequestTriggerReasonsItem]
            ]
        ] = None,
        available_after_time: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[MediaRetrievalListUploadedMediaResponseBody]:
        """
        This endpoint returns a list of all uploaded media (video and still images) matching query parameters, with a maximum query range of one day. Additional media can be retrieved with the [Create a media retrieval request](https://developers.samsara.com/reference/postmediaretrieval) endpoint, and they will be included in the list after they are uploaded. Urls provided by this endpoint expire in 8 hours.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Media Retrieval** under the Safety & Cameras category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        vehicle_ids : str
            A filter on the data based on this comma-separated list of vehicle IDs and externalIds. You can specify up to 20 vehicles. Example: `vehicleIds=1234,5678,samsara.vin:1HGBH41JXMN109186`

        start_time : str
            A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : str
            An end time in RFC 3339 format. End time cannot be more than 24 hours after startTime. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        inputs : typing.Optional[typing.Union[ListUploadedMediaRequestInputsItem, typing.Sequence[ListUploadedMediaRequestInputsItem]]]
            An optional list of desired camera inputs for which to return captured media. If empty, media for all available inputs will be returned.

        media_types : typing.Optional[typing.Union[ListUploadedMediaRequestMediaTypesItem, typing.Sequence[ListUploadedMediaRequestMediaTypesItem]]]
            An optional list of desired media types for which to return captured media. If empty, media for all available media types will be returned. Possible options include: image, videoHighRes.

        trigger_reasons : typing.Optional[typing.Union[ListUploadedMediaRequestTriggerReasonsItem, typing.Sequence[ListUploadedMediaRequestTriggerReasonsItem]]]
            An optional list of desired trigger reasons for which to return captured media. If empty, media for all available trigger reasons will be returned. Possible options include: api, panicButton, periodicStill, rfidEvent, safetyEvent, tripEndStill, tripStartStill, videoRetrieval. videoRetrieval represents media captured for a dashboard video retrieval request.

        available_after_time : typing.Optional[str]
            An optional timestamp in RFC 3339 format that can act as a cursor to track which media has previously been retrieved; only media whose availableAtTime comes after this parameter will be returned. Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[MediaRetrievalListUploadedMediaResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "cameras/media",
            method="GET",
            params={
                "vehicleIds": vehicle_ids,
                "inputs": inputs,
                "mediaTypes": media_types,
                "triggerReasons": trigger_reasons,
                "startTime": start_time,
                "endTime": end_time,
                "availableAfterTime": available_after_time,
                "after": after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    MediaRetrievalListUploadedMediaResponseBody,
                    parse_obj_as(
                        type_=MediaRetrievalListUploadedMediaResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_media_retrieval(
        self, *, retrieval_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[MediaRetrievalGetMediaRetrievalResponseBody]:
        """
        This endpoint returns media information corresponding to a retrieval ID. Retrieval IDs are associated to prior [media retrieval requests](https://developers.samsara.com/reference/postmediaretrieval). Urls provided by this endpoint expire in 8 hours.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Media Retrieval** under the Safety & Cameras category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        retrieval_id : str
            Retrieval ID associated with this media capture request. Examples: 2308cec4-82e0-46f1-8b3c-a3592e5cc21e

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[MediaRetrievalGetMediaRetrievalResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "cameras/media/retrieval",
            method="GET",
            params={
                "retrievalId": retrieval_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    MediaRetrievalGetMediaRetrievalResponseBody,
                    parse_obj_as(
                        type_=MediaRetrievalGetMediaRetrievalResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def post_media_retrieval(
        self,
        *,
        end_time: str,
        inputs: typing.Sequence[MediaRetrievalPostMediaRetrievalRequestBodyInputsItem],
        media_type: MediaRetrievalPostMediaRetrievalRequestBodyMediaType,
        start_time: str,
        vehicle_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[MediaRetrievalPostMediaRetrievalResponseBody]:
        """
        This endpoint creates an asynchronous request to upload certain media from a device. The closest available media to the requested timestamp is returned. Images, videos(both high resolution and low resolution) and hyperlapses are supported.Currently, only unblurred media is supported. If a device is offline, the requested media will be uploaded once it comes back online. Quota limits are enforced for media retrievals made through the API. The Create a media retrieval request response includes information about the media retrieval quota remaining for the organization. The media retrieval quota for the organization is reset at the beginning of each month.The quota is expressed using seconds of High Resolution video. 10 still images are equivalent to a 1 second of High Resolution footage.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Media Retrieval** under the Safety & Cameras category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        end_time : str
            An end time in RFC 3339 format. If endTime is the same as startTime, an image will be captured at startTime. Must be 1 second or more after startTime and no more than the maximum allowed duration per video retrieval type. Please refer to our KB articles for more information. (Examples: 2019-06-13T19:08:55Z, 2019-06-13T19:08:55.455Z, OR 2015-09-15T14:00:42-04:00).

        inputs : typing.Sequence[MediaRetrievalPostMediaRetrievalRequestBodyInputsItem]
            A list of desired camera inputs for which to capture media. Only media with valid inputs (e.g. device has that input stream and device was recording at the time) will be uploaded. An empty list is invalid.

        media_type : MediaRetrievalPostMediaRetrievalRequestBodyMediaType
            The desired media type. If a video is requested, endTime must be after startTime. If an image is requested, endTime must be the same as startTime. Must be one of: image, videoHighRes, videoLowRes. Examples: image, videoHighRes, videoLowRes, hyperlapse.  Valid values: `image`, `videoHighRes`, `videoLowRes`, `hyperlapse`

        start_time : str
            A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        vehicle_id : str
            Vehicle ID for which to initiate media capture. Examples: 1234

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[MediaRetrievalPostMediaRetrievalResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "cameras/media/retrieval",
            method="POST",
            json={
                "endTime": end_time,
                "inputs": inputs,
                "mediaType": media_type,
                "startTime": start_time,
                "vehicleId": vehicle_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    MediaRetrievalPostMediaRetrievalResponseBody,
                    parse_obj_as(
                        type_=MediaRetrievalPostMediaRetrievalResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .driver_tiny_response import DriverTinyResponse
from .hos_logs_list import HosLogsList


class HosLogsForDriver(UniversalBaseModel):
    """
    List of HOS logs for a driver.
    """

    driver: typing.Optional[DriverTinyResponse] = None
    hos_logs: typing_extensions.Annotated[
        typing.Optional[HosLogsList], FieldMetadata(alias="hosLogs"), pydantic.Field(alias="hosLogs")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class DeviceAssetResponseResponseBody(UniversalBaseModel):
    """
    Asset that the device is tied to.
    """

    id: str = pydantic.Field()
    """
    The unique ID of the asset.
    """

    name: str = pydantic.Field()
    """
    The human-readable name of the asset. This is set by a fleet administrator and will appear in both Samsara’s cloud dashboard as well as the Samsara Driver mobile app. By default, this name is the serial number of the Samsara Asset Gateway. It can be set or updated through the Samsara Dashboard or through the API at any time.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .v_1_asset_reefer_response_reefer_stats_alarms_1 import V1AssetReeferResponseReeferStatsAlarms1
from .v_1_asset_reefer_response_reefer_stats_engine_hours import V1AssetReeferResponseReeferStatsEngineHours
from .v_1_asset_reefer_response_reefer_stats_fuel_percentage import V1AssetReeferResponseReeferStatsFuelPercentage
from .v_1_asset_reefer_response_reefer_stats_power_status import V1AssetReeferResponseReeferStatsPowerStatus
from .v_1_asset_reefer_response_reefer_stats_return_air_temp import V1AssetReeferResponseReeferStatsReturnAirTemp
from .v_1_asset_reefer_response_reefer_stats_set_point import V1AssetReeferResponseReeferStatsSetPoint


class V1AssetReeferResponseReeferStats(UniversalBaseModel):
    alarms: typing.Optional[typing.List[V1AssetReeferResponseReeferStatsAlarms1]] = pydantic.Field(default=None)
    """
    Reefer alarms
    """

    engine_hours: typing_extensions.Annotated[
        typing.Optional[typing.List[V1AssetReeferResponseReeferStatsEngineHours]],
        FieldMetadata(alias="engineHours"),
        pydantic.Field(alias="engineHours", description="Engine hours of the reefer"),
    ] = None
    fuel_percentage: typing_extensions.Annotated[
        typing.Optional[typing.List[V1AssetReeferResponseReeferStatsFuelPercentage]],
        FieldMetadata(alias="fuelPercentage"),
        pydantic.Field(alias="fuelPercentage", description="Fuel percentage of the reefer"),
    ] = None
    power_status: typing_extensions.Annotated[
        typing.Optional[typing.List[V1AssetReeferResponseReeferStatsPowerStatus]],
        FieldMetadata(alias="powerStatus"),
        pydantic.Field(alias="powerStatus", description="Power status of the reefer"),
    ] = None
    return_air_temp: typing_extensions.Annotated[
        typing.Optional[typing.List[V1AssetReeferResponseReeferStatsReturnAirTemp]],
        FieldMetadata(alias="returnAirTemp"),
        pydantic.Field(alias="returnAirTemp", description="Return air temperature of the reefer"),
    ] = None
    set_point: typing_extensions.Annotated[
        typing.Optional[typing.List[V1AssetReeferResponseReeferStatsSetPoint]],
        FieldMetadata(alias="setPoint"),
        pydantic.Field(alias="setPoint", description="Set point temperature of the reefer"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



import base64
import hashlib
import hmac


def verify_signature(
    *,
    request_body: str,
    signature_header: str,
    signature_key: str,
    notification_url: str,
) -> bool:
    """
    Verifies and validates an event notification. See the `documentation`_ for more details.

    Args:
        request_body:       The JSON body of the request.
        signature_header:   The value for the `x-samsara-hmacsha256-signature` header.
        signature_key:      The signature key from the Samsara Developer portal for the webhook subscription.
        notification_url:   The notification endpoint URL as defined in the Samsara Developer portal for the webhook
                            subscription.

    Returns:
        bool: True if the signature is valid, indicating that the event can be trusted as it came from Samsara.
            False if the signature validation fails, indicating that the event did not come from Samsara, so it may
            be malicious and should be discarded.

    Raises:
        ValueError: if `signature_key` or `notification_url` are empty.
    """
    if not request_body:
        return False
    if not signature_key:
        raise ValueError("signature_key is empty")
    if not notification_url:
        raise ValueError("notification_url is empty")

    # Perform UTF-8 encoding to bytes
    payload = notification_url + request_body
    payload_bytes = payload.encode("utf-8")
    signature_header_bytes = signature_header.encode("utf-8")
    signature_key_bytes = signature_key.encode("utf-8")

    # Compute the hash value
    hashing_obj = hmac.new(
        key=signature_key_bytes, msg=payload_bytes, digestmod=hashlib.sha256
    )
    hash_bytes = hashing_obj.digest()

    # Compare the computed hash vs the value in the signature header
    hash_base64 = base64.b64encode(hash_bytes)
    return hmac.compare_digest(hash_base64, signature_header_bytes)


# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .goa_vehicle_tiny_response_response_body import GoaVehicleTinyResponseResponseBody
from .ifta_jurisdiction_summary_object_response_body import IftaJurisdictionSummaryObjectResponseBody


class IftaVehicleReportObjectResponseBody(UniversalBaseModel):
    """
    A summary of this vehicle's IFTA data.
    """

    jurisdictions: typing.List[IftaJurisdictionSummaryObjectResponseBody] = pydantic.Field()
    """
    List of jurisdiction summaries.
    """

    vehicle: GoaVehicleTinyResponseResponseBody

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
